<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Models\User;
use Exception;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Str;
use Laravel\Socialite\Facades\Socialite;

class FacebookController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function redirectToFacebook()
    {
        return Socialite::driver('facebook')->redirect();
    }

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function handleFacebookCallback()
    {
        $remember_token = Str::random(64);

        $google2fa = app('pragmarx.google2fa');

        try {

            $user = Socialite::driver('facebook')->user();

            $finduser = User::where('facebook_id', $user->id)->first();

            if ($finduser) {

                Auth::login($finduser);

                return redirect()->intended('user/dashboard');

            } else {
                $newUser = User::create([
                    'name' => $user->name,
                    'email' => $user->email,
                    'facebook_id' => $user->id,
                    'password' => Hash::make($user->email . $user->id),
                    'remember_token' => $remember_token,
                    'google2fa_secret' => $google2fa->generateSecretKey(),
                    'ref_code' => random_strings(10)
                ]);

                $userWalletService = new UserWalletService();
                $userWalletService->create($newUser->id);

                Auth::login($newUser);

                return redirect()->intended('user/dashboard');
            }

        } catch (Exception $e) {
            return redirect(route('login'))->with('error', $e->getMessage());
        }
    }
}
